@@ -19,11 +19,10 @@ from __future__ import annotations
 from typing import Any, Iterable, TypeVar
 
 from marshmallow import ValidationError
-from sqlalchemy import and_, func, or_
+from sqlalchemy import and_, or_, select
 from sqlalchemy.exc import MultipleResultsFound
 from sqlalchemy.orm import Session, joinedload
-from sqlalchemy.orm.query import Query
-from sqlalchemy.sql import ClauseElement
+from sqlalchemy.sql import ClauseElement, Select
 
 from airflow.api_connexion import security
 from airflow.api_connexion.endpoints.request_dict import get_json_request_dict
@@ -49,8 +48,10 @@ from airflow.models.operator import needs_expansion
 from airflow.models.taskinstance import TaskInstance as TI, clear_task_instances
 from airflow.security import permissions
 from airflow.utils.airflow_flask_app import get_airflow_app
+from airflow.utils.db import get_query_count
 from airflow.utils.session import NEW_SESSION, provide_session
-from airflow.utils.state import DagRunState, State
+from airflow.utils.state import DagRunState, TaskInstanceState
+from airflow.www.extensions.init_auth_manager import get_auth_manager
 
 T = TypeVar("T")
 
@@ -72,8 +73,8 @@ def get_task_instance(
 ) -> APIResponse:
     """Get task instance."""
     query = (
-        session.query(TI)
-        .filter(TI.dag_id == dag_id, TI.run_id == dag_run_id, TI.task_id == task_id)
+        select(TI)
+        .where(TI.dag_id == dag_id, TI.run_id == dag_run_id, TI.task_id == task_id)
         .join(TI.dag_run)
         .outerjoin(
             SlaMiss,
@@ -83,12 +84,12 @@ def get_task_instance(
                 SlaMiss.task_id == TI.task_id,
             ),
         )
-        .add_entity(SlaMiss)
+        .add_columns(SlaMiss)
         .options(joinedload(TI.rendered_task_instance_fields))
     )
 
     try:
-        task_instance = query.one_or_none()
+        task_instance = session.execute(query).one_or_none()
     except MultipleResultsFound:
         raise NotFound(
             "Task instance not found", detail="Task instance is mapped, add the map_index value to the URL"
@@ -121,10 +122,8 @@ def get_mapped_task_instance(
 ) -> APIResponse:
     """Get task instance."""
     query = (
-        session.query(TI)
-        .filter(
-            TI.dag_id == dag_id, TI.run_id == dag_run_id, TI.task_id == task_id, TI.map_index == map_index
-        )
+        select(TI)
+        .where(TI.dag_id == dag_id, TI.run_id == dag_run_id, TI.task_id == task_id, TI.map_index == map_index)
         .join(TI.dag_run)
         .outerjoin(
             SlaMiss,
@@ -134,10 +133,11 @@ def get_mapped_task_instance(
                 SlaMiss.task_id == TI.task_id,
             ),
         )
-        .add_entity(SlaMiss)
+        .add_columns(SlaMiss)
         .options(joinedload(TI.rendered_task_instance_fields))
     )
-    task_instance = query.one_or_none()
+    task_instance = session.execute(query).one_or_none()
+
     if task_instance is None:
         raise NotFound("Task instance not found")
 
@@ -152,6 +152,8 @@ def get_mapped_task_instance(
         "start_date_lte": format_datetime,
         "end_date_gte": format_datetime,
         "end_date_lte": format_datetime,
+        "updated_at_gte": format_datetime,
+        "updated_at_lte": format_datetime,
     },
 )
 @security.requires_access(
@@ -173,6 +175,8 @@ def get_mapped_task_instances(
     start_date_lte: str | None = None,
     end_date_gte: str | None = None,
     end_date_lte: str | None = None,
+    updated_at_gte: str | None = None,
+    updated_at_lte: str | None = None,
     duration_gte: float | None = None,
     duration_lte: float | None = None,
     state: list[str] | None = None,
@@ -185,16 +189,17 @@ def get_mapped_task_instances(
 ) -> APIResponse:
     """Get list of task instances."""
     # Because state can be 'none'
-    states = _convert_state(state)
+    states = _convert_ti_states(state)
 
     base_query = (
-        session.query(TI)
-        .filter(TI.dag_id == dag_id, TI.run_id == dag_run_id, TI.task_id == task_id, TI.map_index >= 0)
+        select(TI)
+        .where(TI.dag_id == dag_id, TI.run_id == dag_run_id, TI.task_id == task_id, TI.map_index >= 0)
         .join(TI.dag_run)
     )
 
     # 0 can mean a mapped TI that expanded to an empty list, so it is not an automatic 404
-    if base_query.with_entities(func.count("*")).scalar() == 0:
+    unfiltered_total_count = get_query_count(base_query, session=session)
+    if unfiltered_total_count == 0:
         dag = get_airflow_app().dag_bag.get_dag(dag_id)
         if not dag:
             error_message = f"DAG {dag_id} not found"
@@ -208,73 +213,78 @@ def get_mapped_task_instances(
             raise NotFound(error_message)
 
     # Other search criteria
-    query = _apply_range_filter(
+    base_query = _apply_range_filter(
         base_query,
         key=DR.execution_date,
         value_range=(execution_date_gte, execution_date_lte),
     )
-    query = _apply_range_filter(query, key=TI.start_date, value_range=(start_date_gte, start_date_lte))
-    query = _apply_range_filter(query, key=TI.end_date, value_range=(end_date_gte, end_date_lte))
-    query = _apply_range_filter(query, key=TI.duration, value_range=(duration_gte, duration_lte))
-    query = _apply_array_filter(query, key=TI.state, values=states)
-    query = _apply_array_filter(query, key=TI.pool, values=pool)
-    query = _apply_array_filter(query, key=TI.queue, values=queue)
+    base_query = _apply_range_filter(
+        base_query, key=TI.start_date, value_range=(start_date_gte, start_date_lte)
+    )
+    base_query = _apply_range_filter(base_query, key=TI.end_date, value_range=(end_date_gte, end_date_lte))
+    base_query = _apply_range_filter(base_query, key=TI.duration, value_range=(duration_gte, duration_lte))
+    base_query = _apply_range_filter(
+        base_query, key=TI.updated_at, value_range=(updated_at_gte, updated_at_lte)
+    )
+    base_query = _apply_array_filter(base_query, key=TI.state, values=states)
+    base_query = _apply_array_filter(base_query, key=TI.pool, values=pool)
+    base_query = _apply_array_filter(base_query, key=TI.queue, values=queue)
 
     # Count elements before joining extra columns
-    total_entries = query.with_entities(func.count("*")).scalar()
+    total_entries = get_query_count(base_query, session=session)
 
     # Add SLA miss
-    query = (
-        query.join(
+    entry_query = (
+        base_query.outerjoin(
             SlaMiss,
             and_(
                 SlaMiss.dag_id == TI.dag_id,
                 SlaMiss.task_id == TI.task_id,
                 SlaMiss.execution_date == DR.execution_date,
             ),
-            isouter=True,
         )
-        .add_entity(SlaMiss)
+        .add_columns(SlaMiss)
         .options(joinedload(TI.rendered_task_instance_fields))
     )
 
     if order_by:
         if order_by == "state":
-            query = query.order_by(TI.state.asc(), TI.map_index.asc())
+            entry_query = entry_query.order_by(TI.state.asc(), TI.map_index.asc())
         elif order_by == "-state":
-            query = query.order_by(TI.state.desc(), TI.map_index.asc())
+            entry_query = entry_query.order_by(TI.state.desc(), TI.map_index.asc())
         elif order_by == "-map_index":
-            query = query.order_by(TI.map_index.desc())
+            entry_query = entry_query.order_by(TI.map_index.desc())
         else:
             raise BadRequest(detail=f"Ordering with '{order_by}' is not supported")
     else:
-        query = query.order_by(TI.map_index.asc())
+        entry_query = entry_query.order_by(TI.map_index.asc())
 
-    task_instances = query.offset(offset).limit(limit).all()
+    # using execute because we want the SlaMiss entity. Scalars don't return None for missing entities
+    task_instances = session.execute(entry_query.offset(offset).limit(limit)).all()
     return task_instance_collection_schema.dump(
         TaskInstanceCollection(task_instances=task_instances, total_entries=total_entries)
     )
 
 
-def _convert_state(states: Iterable[str] | None) -> list[str | None] | None:
+def _convert_ti_states(states: Iterable[str] | None) -> list[TaskInstanceState | None] | None:
     if not states:
         return None
-    return [State.NONE if s == "none" else s for s in states]
+    return [None if s in ("none", None) else TaskInstanceState(s) for s in states]
 
 
-def _apply_array_filter(query: Query, key: ClauseElement, values: Iterable[Any] | None) -> Query:
+def _apply_array_filter(query: Select, key: ClauseElement, values: Iterable[Any] | None) -> Select:
     if values is not None:
         cond = ((key == v) for v in values)
-        query = query.filter(or_(*cond))
+        query = query.where(or_(*cond))
     return query
 
 
-def _apply_range_filter(query: Query, key: ClauseElement, value_range: tuple[T, T]) -> Query:
+def _apply_range_filter(query: Select, key: ClauseElement, value_range: tuple[T, T]) -> Select:
     gte_value, lte_value = value_range
     if gte_value is not None:
-        query = query.filter(key >= gte_value)
+        query = query.where(key >= gte_value)
     if lte_value is not None:
-        query = query.filter(key <= lte_value)
+        query = query.where(key <= lte_value)
     return query
 
 
@@ -286,6 +296,8 @@ def _apply_range_filter(query: Query, key: ClauseElement, value_range: tuple[T,
         "start_date_lte": format_datetime,
         "end_date_gte": format_datetime,
         "end_date_lte": format_datetime,
+        "updated_at_gte": format_datetime,
+        "updated_at_lte": format_datetime,
     },
 )
 @security.requires_access(
@@ -307,6 +319,8 @@ def get_task_instances(
     start_date_lte: str | None = None,
     end_date_gte: str | None = None,
     end_date_lte: str | None = None,
+    updated_at_gte: str | None = None,
+    updated_at_lte: str | None = None,
     duration_gte: float | None = None,
     duration_lte: float | None = None,
     state: list[str] | None = None,
@@ -317,14 +331,14 @@ def get_task_instances(
 ) -> APIResponse:
     """Get list of task instances."""
     # Because state can be 'none'
-    states = _convert_state(state)
+    states = _convert_ti_states(state)
 
-    base_query = session.query(TI).join(TI.dag_run)
+    base_query = select(TI).join(TI.dag_run)
 
     if dag_id != "~":
-        base_query = base_query.filter(TI.dag_id == dag_id)
+        base_query = base_query.where(TI.dag_id == dag_id)
     if dag_run_id != "~":
-        base_query = base_query.filter(TI.run_id == dag_run_id)
+        base_query = base_query.where(TI.run_id == dag_run_id)
     base_query = _apply_range_filter(
         base_query,
         key=DR.execution_date,
@@ -335,27 +349,33 @@ def get_task_instances(
     )
     base_query = _apply_range_filter(base_query, key=TI.end_date, value_range=(end_date_gte, end_date_lte))
     base_query = _apply_range_filter(base_query, key=TI.duration, value_range=(duration_gte, duration_lte))
+    base_query = _apply_range_filter(
+        base_query, key=TI.updated_at, value_range=(updated_at_gte, updated_at_lte)
+    )
     base_query = _apply_array_filter(base_query, key=TI.state, values=states)
     base_query = _apply_array_filter(base_query, key=TI.pool, values=pool)
     base_query = _apply_array_filter(base_query, key=TI.queue, values=queue)
 
     # Count elements before joining extra columns
-    total_entries = base_query.with_entities(func.count("*")).scalar()
+    total_entries = get_query_count(base_query, session=session)
+
     # Add join
-    query = (
-        base_query.join(
+    entry_query = (
+        base_query.outerjoin(
             SlaMiss,
             and_(
                 SlaMiss.dag_id == TI.dag_id,
                 SlaMiss.task_id == TI.task_id,
                 SlaMiss.execution_date == DR.execution_date,
             ),
-            isouter=True,
         )
-        .add_entity(SlaMiss)
+        .add_columns(SlaMiss)
         .options(joinedload(TI.rendered_task_instance_fields))
+        .offset(offset)
+        .limit(limit)
     )
-    task_instances = query.offset(offset).limit(limit).all()
+    # using execute because we want the SlaMiss entity. Scalars don't return None for missing entities
+    task_instances = session.execute(entry_query).all()
     return task_instance_collection_schema.dump(
         TaskInstanceCollection(task_instances=task_instances, total_entries=total_entries)
     )
@@ -376,10 +396,12 @@ def get_task_instances_batch(session: Session = NEW_SESSION) -> APIResponse:
         data = task_instance_batch_form.load(body)
     except ValidationError as err:
         raise BadRequest(detail=str(err.messages))
-    states = _convert_state(data["state"])
-    base_query = session.query(TI).join(TI.dag_run)
+    states = _convert_ti_states(data["state"])
+    base_query = select(TI).join(TI.dag_run)
 
     base_query = _apply_array_filter(base_query, key=TI.dag_id, values=data["dag_ids"])
+    base_query = _apply_array_filter(base_query, key=TI.run_id, values=data["dag_run_ids"])
+    base_query = _apply_array_filter(base_query, key=TI.task_id, values=data["task_ids"])
     base_query = _apply_range_filter(
         base_query,
         key=DR.execution_date,
@@ -401,7 +423,7 @@ def get_task_instances_batch(session: Session = NEW_SESSION) -> APIResponse:
     base_query = _apply_array_filter(base_query, key=TI.queue, values=data["queue"])
 
     # Count elements before joining extra columns
-    total_entries = base_query.with_entities(func.count("*")).scalar()
+    total_entries = get_query_count(base_query, session=session)
     # Add join
     base_query = base_query.join(
         SlaMiss,
@@ -411,9 +433,10 @@ def get_task_instances_batch(session: Session = NEW_SESSION) -> APIResponse:
             SlaMiss.execution_date == DR.execution_date,
         ),
         isouter=True,
-    ).add_entity(SlaMiss)
+    ).add_columns(SlaMiss)
     ti_query = base_query.options(joinedload(TI.rendered_task_instance_fields))
-    task_instances = ti_query.all()
+    # using execute because we want the SlaMiss entity. Scalars don't return None for missing entities
+    task_instances = session.execute(ti_query).all()
 
     return task_instance_collection_schema.dump(
         TaskInstanceCollection(task_instances=task_instances, total_entries=total_entries)
@@ -449,9 +472,7 @@ def post_clear_task_instances(*, dag_id: str, session: Session = NEW_SESSION) ->
     downstream = data.pop("include_downstream", False)
     upstream = data.pop("include_upstream", False)
     if dag_run_id is not None:
-        dag_run: DR | None = (
-            session.query(DR).filter(DR.dag_id == dag_id, DR.run_id == dag_run_id).one_or_none()
-        )
+        dag_run: DR | None = session.scalar(select(DR).where(DR.dag_id == dag_id, DR.run_id == dag_run_id))
         if dag_run is None:
             error_message = f"Dag Run id {dag_run_id} not found in dag {dag_id}"
             raise NotFound(error_message)
@@ -474,16 +495,17 @@ def post_clear_task_instances(*, dag_id: str, session: Session = NEW_SESSION) ->
             # If we had upstream/downstream etc then also include those!
             task_ids.extend(tid for tid in dag.task_dict if tid != task_id)
     task_instances = dag.clear(dry_run=True, dag_bag=get_airflow_app().dag_bag, task_ids=task_ids, **data)
+
     if not dry_run:
         clear_task_instances(
-            task_instances.all(),
+            task_instances,
             session,
             dag=dag,
             dag_run_state=DagRunState.QUEUED if reset_dag_runs else False,
         )
 
     return task_instance_reference_collection_schema.dump(
-        TaskInstanceReferenceCollection(task_instances=task_instances.all())
+        TaskInstanceReferenceCollection(task_instances=task_instances)
     )
 
 
@@ -520,9 +542,11 @@ def post_set_task_instances_state(*, dag_id: str, session: Session = NEW_SESSION
     if (
         execution_date
         and (
-            session.query(TI)
-            .filter(TI.task_id == task_id, TI.dag_id == dag_id, TI.execution_date == execution_date)
-            .one_or_none()
+            session.scalars(
+                select(TI).where(
+                    TI.task_id == task_id, TI.dag_id == dag_id, TI.execution_date == execution_date
+                )
+            ).one_or_none()
         )
         is None
     ):
@@ -530,8 +554,8 @@ def post_set_task_instances_state(*, dag_id: str, session: Session = NEW_SESSION
             detail=f"Task instance not found for task {task_id!r} on execution_date {execution_date}"
         )
 
-    if run_id and not session.query(TI).get(
-        {"task_id": task_id, "dag_id": dag_id, "run_id": run_id, "map_index": -1}
+    if run_id and not session.get(
+        TI, {"task_id": task_id, "dag_id": dag_id, "run_id": run_id, "map_index": -1}
     ):
         error_message = f"Task instance not found for task {task_id!r} on DAG run with ID {run_id!r}"
         raise NotFound(detail=error_message)
@@ -583,8 +607,8 @@ def patch_task_instance(
     if not dag.has_task(task_id):
         raise NotFound("Task not found", detail=f"Task {task_id!r} not found in DAG {dag_id!r}")
 
-    ti: TI | None = session.query(TI).get(
-        {"task_id": task_id, "dag_id": dag_id, "run_id": dag_run_id, "map_index": map_index}
+    ti: TI | None = session.get(
+        TI, {"task_id": task_id, "dag_id": dag_id, "run_id": dag_run_id, "map_index": map_index}
     )
 
     if not ti:
@@ -641,8 +665,8 @@ def set_task_instance_note(
         raise BadRequest(detail=str(err))
 
     query = (
-        session.query(TI)
-        .filter(TI.dag_id == dag_id, TI.run_id == dag_run_id, TI.task_id == task_id)
+        select(TI)
+        .where(TI.dag_id == dag_id, TI.run_id == dag_run_id, TI.task_id == task_id)
         .join(TI.dag_run)
         .outerjoin(
             SlaMiss,
@@ -652,16 +676,16 @@ def set_task_instance_note(
                 SlaMiss.task_id == TI.task_id,
             ),
         )
-        .add_entity(SlaMiss)
+        .add_columns(SlaMiss)
         .options(joinedload(TI.rendered_task_instance_fields))
     )
     if map_index == -1:
-        query = query.filter(or_(TI.map_index == -1, TI.map_index is None))
+        query = query.where(or_(TI.map_index == -1, TI.map_index is None))
     else:
-        query = query.filter(TI.map_index == map_index)
+        query = query.where(TI.map_index == map_index)
 
     try:
-        result = query.one_or_none()
+        result = session.execute(query).one_or_none()
     except MultipleResultsFound:
         raise NotFound(
             "Task instance not found", detail="Task instance is mapped, add the map_index value to the URL"
@@ -671,9 +695,8 @@ def set_task_instance_note(
         raise NotFound(error_message)
 
     ti, sla_miss = result
-    from flask_login import current_user
 
-    current_user_id = getattr(current_user, "id", None)
+    current_user_id = get_auth_manager().get_user_id()
     if ti.task_instance_note is None:
         ti.note = (new_note, current_user_id)
     else:
