@@ -14,7 +14,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""This package is deprecated. Please use `airflow.utils.log`."""
+"""This package is deprecated. Please use :mod:`airflow.utils.log`."""
 from __future__ import annotations
 
 import warnings
