@@ -16,20 +16,24 @@
 # specific language governing permissions and limitations
 # under the License.
 """
-DebugExecutor
+DebugExecutor.
 
 .. seealso::
     For more information on how the DebugExecutor works, take a look at the guide:
     :ref:`executor:DebugExecutor`
 """
+from __future__ import annotations
 
 import threading
-from typing import Any, Dict, List, Optional
+import time
+from typing import TYPE_CHECKING, Any
 
-from airflow.configuration import conf
 from airflow.executors.base_executor import BaseExecutor
-from airflow.models.taskinstance import TaskInstance, TaskInstanceKey
-from airflow.utils.state import State
+from airflow.utils.state import TaskInstanceState
+
+if TYPE_CHECKING:
+    from airflow.models.taskinstance import TaskInstance
+    from airflow.models.taskinstancekey import TaskInstanceKey
 
 
 class DebugExecutor(BaseExecutor):
@@ -42,11 +46,18 @@ class DebugExecutor(BaseExecutor):
 
     _terminated = threading.Event()
 
+    is_single_threaded: bool = True
+    is_production: bool = False
+
+    change_sensor_mode_to_reschedule: bool = True
+
     def __init__(self):
         super().__init__()
-        self.tasks_to_run: List[TaskInstance] = []
+        self.tasks_to_run: list[TaskInstance] = []
         # Place where we keep information for task instance raw run
-        self.tasks_params: Dict[TaskInstanceKey, Dict[str, Any]] = {}
+        self.tasks_params: dict[TaskInstanceKey, dict[str, Any]] = {}
+        from airflow.configuration import conf
+
         self.fail_fast = conf.getboolean("debug", "fail_fast")
 
     def execute_async(self, *args, **kwargs) -> None:
@@ -57,16 +68,15 @@ class DebugExecutor(BaseExecutor):
         while self.tasks_to_run:
             ti = self.tasks_to_run.pop(0)
             if self.fail_fast and not task_succeeded:
-                self.log.info("Setting %s to %s", ti.key, State.UPSTREAM_FAILED)
-                ti.set_state(State.UPSTREAM_FAILED)
-                self.change_state(ti.key, State.UPSTREAM_FAILED)
+                self.log.info("Setting %s to %s", ti.key, TaskInstanceState.UPSTREAM_FAILED)
+                ti.set_state(TaskInstanceState.UPSTREAM_FAILED)
+                self.change_state(ti.key, TaskInstanceState.UPSTREAM_FAILED)
                 continue
 
             if self._terminated.is_set():
-                self.log.info("Executor is terminated! Stopping %s to %s", ti.key, State.FAILED)
-                ti.set_state(State.FAILED)
-                self.change_state(ti.key, State.FAILED)
-                ti._run_finished_callback()
+                self.log.info("Executor is terminated! Stopping %s to %s", ti.key, TaskInstanceState.FAILED)
+                ti.set_state(TaskInstanceState.FAILED)
+                self.change_state(ti.key, TaskInstanceState.FAILED)
                 continue
 
             task_succeeded = self._run_task(ti)
@@ -76,14 +86,12 @@ class DebugExecutor(BaseExecutor):
         key = ti.key
         try:
             params = self.tasks_params.pop(ti.key, {})
-            ti._run_raw_task(job_id=ti.job_id, **params)
-            self.change_state(key, State.SUCCESS)
-            ti._run_finished_callback()
+            ti.run(job_id=ti.job_id, **params)
+            self.change_state(key, TaskInstanceState.SUCCESS)
             return True
         except Exception as e:
-            ti.set_state(State.FAILED)
-            self.change_state(key, State.FAILED)
-            ti._run_finished_callback(error=e)
+            ti.set_state(TaskInstanceState.FAILED)
+            self.change_state(key, TaskInstanceState.FAILED)
             self.log.exception("Failed to execute task: %s.", str(e))
             return False
 
@@ -91,13 +99,14 @@ class DebugExecutor(BaseExecutor):
         self,
         task_instance: TaskInstance,
         mark_success: bool = False,
-        pickle_id: Optional[str] = None,
+        pickle_id: int | None = None,
         ignore_all_deps: bool = False,
         ignore_depends_on_past: bool = False,
+        wait_for_past_depends_before_skipping: bool = False,
         ignore_task_deps: bool = False,
         ignore_ti_state: bool = False,
-        pool: Optional[str] = None,
-        cfg_path: Optional[str] = None,
+        pool: str | None = None,
+        cfg_path: str | None = None,
     ) -> None:
         """Queues task instance with empty command because we do not need it."""
         self.queue_command(
@@ -114,11 +123,17 @@ class DebugExecutor(BaseExecutor):
 
     def trigger_tasks(self, open_slots: int) -> None:
         """
-        Triggers tasks. Instead of calling exec_async we just
-        add task instance to tasks_to_run queue.
+        Triggers tasks.
+
+        Instead of calling exec_async we just add task instance to tasks_to_run queue.
 
         :param open_slots: Number of open slots
         """
+        if not self.queued_tasks:
+            # wait a bit if there are no tasks ready to be executed to avoid spinning too fast in the void
+            time.sleep(0.5)
+            return
+
         sorted_queue = sorted(
             self.queued_tasks.items(),
             key=lambda x: x[1][1],
@@ -131,19 +146,16 @@ class DebugExecutor(BaseExecutor):
             self.tasks_to_run.append(ti)  # type: ignore
 
     def end(self) -> None:
-        """
-        When the method is called we just set states of queued tasks
-        to UPSTREAM_FAILED marking them as not executed.
-        """
+        """Set states of queued tasks to UPSTREAM_FAILED marking them as not executed."""
         for ti in self.tasks_to_run:
-            self.log.info("Setting %s to %s", ti.key, State.UPSTREAM_FAILED)
-            ti.set_state(State.UPSTREAM_FAILED)
-            self.change_state(ti.key, State.UPSTREAM_FAILED)
+            self.log.info("Setting %s to %s", ti.key, TaskInstanceState.UPSTREAM_FAILED)
+            ti.set_state(TaskInstanceState.UPSTREAM_FAILED)
+            self.change_state(ti.key, TaskInstanceState.UPSTREAM_FAILED)
 
     def terminate(self) -> None:
         self._terminated.set()
 
-    def change_state(self, key: TaskInstanceKey, state: str, info=None) -> None:
+    def change_state(self, key: TaskInstanceKey, state: TaskInstanceState, info=None) -> None:
         self.log.debug("Popping %s from executor task queue.", key)
         self.running.remove(key)
         self.event_buffer[key] = state, info
