@@ -28,7 +28,7 @@ from collections import deque
 from contextlib import suppress
 from copy import copy
 from queue import SimpleQueue
-from typing import TYPE_CHECKING, Deque
+from typing import TYPE_CHECKING
 
 from sqlalchemy import func
 
@@ -83,14 +83,12 @@ DISABLE_LISTENER = conf.getboolean("logging", "disable_trigger_handler_queue_lis
 
 def configure_trigger_log_handler():
     """
-    Configure logging such that each trigger logs to its own file and
-    can be exposed through the airflow webserver.
+    Configure logging where each trigger logs to its own file and can be exposed via the airflow webserver.
 
     Generally speaking, we take the log handler configured for logger ``airflow.task``,
     wrap it with TriggerHandlerWrapper, and set it as the handler for root logger.
 
-    If there already is a handler configured for the root logger
-    and it supports triggers, we wrap it instead.
+    If there already is a handler configured for the root logger and it supports triggers, we wrap it instead.
 
     :meta private:
     """
@@ -237,9 +235,7 @@ def setup_queue_listener():
 
 class TriggererJobRunner(BaseJobRunner["Job | JobPydantic"], LoggingMixin):
     """
-    TriggererJobRunner continuously runs active triggers in asyncio, watching
-    for them to fire off their events and then dispatching that information
-    to their dependent tasks/DAGs.
+    Run active triggers in asyncio and update their dependent tests/DAGs once their events have fired.
 
     It runs as two threads:
      - The main thread does DB calls/checkins
@@ -261,6 +257,8 @@ class TriggererJobRunner(BaseJobRunner["Job | JobPydantic"], LoggingMixin):
         else:
             raise ValueError(f"Capacity number {capacity} is invalid")
 
+        self.health_check_threshold = conf.getint("triggerer", "triggerer_health_check_threshold")
+
         should_queue = True
         if DISABLE_WRAPPER:
             self.log.warning(
@@ -291,16 +289,17 @@ class TriggererJobRunner(BaseJobRunner["Job | JobPydantic"], LoggingMixin):
     @provide_session
     def is_needed(cls, session) -> bool:
         """
-        Tests if the triggerer job needs to be run (i.e., if there are triggers
-        in the trigger table).
+        Test if the triggerer job needs to be run (i.e., if there are triggers in the trigger table).
+
         This is used for the warning boxes in the UI.
         """
         return session.query(func.count(Trigger.id)).scalar() > 0
 
     def on_kill(self):
         """
-        Called when there is an external kill command (via the heartbeat
-        mechanism, for example)
+        Stop the trigger runner.
+
+        Called when there is an external kill command (via the heartbeat mechanism, for example).
         """
         self.trigger_runner.stop = True
 
@@ -311,7 +310,7 @@ class TriggererJobRunner(BaseJobRunner["Job | JobPydantic"], LoggingMixin):
             self.listener.stop()
 
     def _exit_gracefully(self, signum, frame) -> None:
-        """Helper method to clean up processor_agent to avoid leaving orphan processes."""
+        """Clean up processor_agent to avoid leaving orphan processes."""
         # The first time, try to exit nicely
         if not self.trigger_runner.stop:
             self.log.info("Exiting gracefully upon receiving signal %s", signum)
@@ -345,11 +344,7 @@ class TriggererJobRunner(BaseJobRunner["Job | JobPydantic"], LoggingMixin):
         return None
 
     def _run_trigger_loop(self) -> None:
-        """
-        The main-thread trigger loop.
-
-        This runs synchronously and handles all database reads/writes.
-        """
+        """Run synchronously and handle all database reads/writes; the main-thread trigger loop."""
         while not self.trigger_runner.stop:
             if not self.trigger_runner.is_alive():
                 self.log.error("Trigger runner thread has died! Exiting.")
@@ -369,20 +364,13 @@ class TriggererJobRunner(BaseJobRunner["Job | JobPydantic"], LoggingMixin):
             time.sleep(1)
 
     def load_triggers(self):
-        """
-        Queries the database to get the triggers we're supposed to be running,
-        adds them to our runner, and then removes ones from it we no longer
-        need.
-        """
-        Trigger.assign_unassigned(self.job.id, self.capacity, self.job.heartrate)
+        """Query the database for the triggers we're supposed to be running and update the runner."""
+        Trigger.assign_unassigned(self.job.id, self.capacity, self.health_check_threshold)
         ids = Trigger.ids_for_triggerer(self.job.id)
         self.trigger_runner.update_triggers(set(ids))
 
     def handle_events(self):
-        """
-        Handles outbound events from triggers - dispatching them into the Trigger
-        model where they are then pushed into the relevant task instances.
-        """
+        """Dispatch outbound events to the Trigger model which pushes them to the relevant task instances."""
         while self.trigger_runner.events:
             # Get the event and its trigger ID
             trigger_id, event = self.trigger_runner.events.popleft()
@@ -393,8 +381,9 @@ class TriggererJobRunner(BaseJobRunner["Job | JobPydantic"], LoggingMixin):
 
     def handle_failed_triggers(self):
         """
-        Handles "failed" triggers - ones that errored or exited before they
-        sent an event. Task Instances that depend on them need failing.
+        Handle "failed" triggers. - ones that errored or exited before they sent an event.
+
+        Task Instances that depend on them need failing.
         """
         while self.trigger_runner.failed_triggers:
             # Tell the model to fail this trigger's deps
@@ -404,7 +393,10 @@ class TriggererJobRunner(BaseJobRunner["Job | JobPydantic"], LoggingMixin):
             Stats.incr("triggers.failed")
 
     def emit_metrics(self):
-        Stats.gauge("triggers.running", len(self.trigger_runner.triggers))
+        Stats.gauge(f"triggers.running.{self.job.hostname}", len(self.trigger_runner.triggers))
+        Stats.gauge(
+            "triggers.running", len(self.trigger_runner.triggers), tags={"hostname": self.job.hostname}
+        )
 
 
 class TriggerDetails(TypedDict):
@@ -432,16 +424,16 @@ class TriggerRunner(threading.Thread, LoggingMixin):
     trigger_cache: dict[str, type[BaseTrigger]]
 
     # Inbound queue of new triggers
-    to_create: Deque[tuple[int, BaseTrigger]]
+    to_create: deque[tuple[int, BaseTrigger]]
 
     # Inbound queue of deleted triggers
-    to_cancel: Deque[int]
+    to_cancel: deque[int]
 
     # Outbound queue of events
-    events: Deque[tuple[int, TriggerEvent]]
+    events: deque[tuple[int, TriggerEvent]]
 
     # Outbound queue of failed triggers
-    failed_triggers: Deque[tuple[int, BaseException]]
+    failed_triggers: deque[tuple[int, BaseException]]
 
     # Should-we-stop flag
     stop: bool = False
@@ -457,15 +449,14 @@ class TriggerRunner(threading.Thread, LoggingMixin):
         self.job_id = None
 
     def run(self):
-        """Sync entrypoint - just runs arun in an async loop."""
+        """Sync entrypoint - just run a run in an async loop."""
         asyncio.run(self.arun())
 
     async def arun(self):
         """
-        Main (asynchronous) logic loop.
+        Run trigger addition/deletion/cleanup; main (asynchronous) logic loop.
 
-        The loop in here runs trigger addition/deletion/cleanup. Actual
-        triggers run in their own separate coroutines.
+        Actual triggers run in their own separate coroutines.
         """
         watchdog = asyncio.create_task(self.block_watchdog())
         last_status = time.time()
@@ -489,10 +480,7 @@ class TriggerRunner(threading.Thread, LoggingMixin):
         await watchdog
 
     async def create_triggers(self):
-        """
-        Drain the to_create queue and create all triggers that have been
-        requested in the DB that we don't yet have.
-        """
+        """Drain the to_create queue and create all new triggers that have been requested in the DB."""
         while self.to_create:
             trigger_id, trigger_instance = self.to_create.popleft()
             if trigger_id not in self.triggers:
@@ -513,8 +501,9 @@ class TriggerRunner(threading.Thread, LoggingMixin):
 
     async def cancel_triggers(self):
         """
-        Drain the to_cancel queue and ensure all triggers that are not in the
-        DB are cancelled, so the cleanup job deletes them.
+        Drain the to_cancel queue and ensure all triggers that are not in the DB are cancelled.
+
+        This allows the cleanup job to delete them.
         """
         while self.to_cancel:
             trigger_id = self.to_cancel.popleft()
@@ -525,9 +514,9 @@ class TriggerRunner(threading.Thread, LoggingMixin):
 
     async def cleanup_finished_triggers(self):
         """
-        Go through all trigger tasks (coroutines) and clean up entries for
-        ones that have exited, optionally warning users if the exit was
-        not normal.
+        Go through all trigger tasks (coroutines) and clean up entries for ones that have exited.
+
+        Optionally warn users if the exit was not normal.
         """
         for trigger_id, details in list(self.triggers.items()):
             if details["task"].done():
@@ -581,7 +570,7 @@ class TriggerRunner(threading.Thread, LoggingMixin):
             # be a busy event loop.
             time_elapsed = time.monotonic() - last_run
             if time_elapsed > 0.2:
-                self.log.error(
+                self.log.info(
                     "Triggerer's async thread was blocked for %.2f seconds, "
                     "likely by a badly-written trigger. Set PYTHONASYNCIODEBUG=1 "
                     "to get more information on overrunning coroutines.",
@@ -591,10 +580,7 @@ class TriggerRunner(threading.Thread, LoggingMixin):
 
     @staticmethod
     def set_individual_trigger_logging(trigger):
-        """
-        Setting these context vars allows log messages for individual triggers
-        to be routed to distinct files and filtered from triggerer stdout.
-        """
+        """Configure trigger logging to allow individual files and stdout filtering."""
         # set logging context vars for routing to appropriate handler
         ctx_task_instance.set(trigger.task_instance)
         ctx_trigger_id.set(trigger.trigger_id)
@@ -604,10 +590,7 @@ class TriggerRunner(threading.Thread, LoggingMixin):
         ctx_indiv_trigger.set(True)
 
     async def run_trigger(self, trigger_id, trigger):
-        """
-        Wrapper which runs an actual trigger (they are async generators)
-        and pushes their events into our outbound event deque.
-        """
+        """Run a trigger (they are async generators) and push their events into our outbound event deque."""
         name = self.triggers[trigger_id]["name"]
         self.log.info("trigger %s starting", name)
         try:
@@ -617,8 +600,7 @@ class TriggerRunner(threading.Thread, LoggingMixin):
                 self.triggers[trigger_id]["events"] += 1
                 self.events.append((trigger_id, event))
         except asyncio.CancelledError as err:
-            timeout = trigger.task_instance.trigger_timeout
-            if timeout:
+            if timeout := trigger.task_instance.trigger_timeout:
                 timeout = timeout.replace(tzinfo=timezone.utc) if not timeout.tzinfo else timeout
                 if timeout < timezone.utcnow():
                     self.log.error("Trigger cancelled due to timeout")
@@ -651,8 +633,7 @@ class TriggerRunner(threading.Thread, LoggingMixin):
 
     def update_triggers(self, requested_trigger_ids: set[int]):
         """
-        Called from the main thread to request that we update what
-        triggers we're running.
+        Request that we update what triggers we're running.
 
         Works out the differences - ones to add, and ones to remove - then
         adds them to the deques so the subthread can actually mutate the running
@@ -704,7 +685,7 @@ class TriggerRunner(threading.Thread, LoggingMixin):
 
     def set_trigger_logging_metadata(self, ti: TaskInstance, trigger_id, trigger):
         """
-        Set up logging for triggers
+        Set up logging for triggers.
 
         We want to ensure that each trigger logs to its own file and that the log messages are not
         propagated to parent loggers.
@@ -719,7 +700,7 @@ class TriggerRunner(threading.Thread, LoggingMixin):
 
     def get_trigger_by_classpath(self, classpath: str) -> type[BaseTrigger]:
         """
-        Gets a trigger class by its classpath ("path.to.module.classname").
+        Get a trigger class by its classpath ("path.to.module.classname").
 
         Uses a cache dictionary to speed up lookups after the first time.
         """
