@@ -23,7 +23,7 @@ from typing import Any
 
 
 def get_python_source(x: Any) -> str | None:
-    """Helper function to get Python source (or not), preventing exceptions"""
+    """Get Python source (or not), preventing exceptions."""
     if isinstance(x, str):
         return x
 
@@ -80,7 +80,7 @@ def prepare_code_snippet(file_path: str, line_no: int, context_lines_count: int
 
 
 def get_terminal_formatter(**opts):
-    """Returns the best formatter available in the current terminal."""
+    """Return the best formatter available in the current terminal."""
     if "256" in os.environ.get("TERM", ""):
         from pygments.formatters.terminal256 import Terminal256Formatter
 
