@@ -37,7 +37,8 @@ from airflow.utils.task_group import TaskGroup
 
 def _refine_color(color: str):
     """
-    Converts color in #RGB (12 bits) format to #RRGGBB (32 bits), if it possible.
+    Convert color in #RGB (12 bits) format to #RRGGBB (32 bits), if it possible.
+
     Otherwise, it returns the original value. Graphviz does not support colors in #RGB format.
 
     :param color: Text representation of color
@@ -56,9 +57,9 @@ def _draw_task(
     parent_graph: graphviz.Digraph,
     states_by_task_id: dict[Any, Any] | None,
 ) -> None:
-    """Draw a single task on the given parent_graph"""
+    """Draw a single task on the given parent_graph."""
     if states_by_task_id:
-        state = states_by_task_id.get(task.task_id, State.NONE)
+        state = states_by_task_id.get(task.task_id)
         color = State.color_fg(state)
         fill_color = State.color(state)
     else:
@@ -80,7 +81,7 @@ def _draw_task(
 def _draw_task_group(
     task_group: TaskGroup, parent_graph: graphviz.Digraph, states_by_task_id: dict[str, str] | None
 ) -> None:
-    """Draw the given task_group and its children on the given parent_graph"""
+    """Draw the given task_group and its children on the given parent_graph."""
     # Draw joins
     if task_group.upstream_group_ids or task_group.upstream_task_ids:
         parent_graph.node(
@@ -143,11 +144,10 @@ def _draw_nodes(
 
 def render_dag_dependencies(deps: dict[str, list[DagDependency]]) -> graphviz.Digraph:
     """
-    Renders the DAG dependency to the DOT object.
+    Render the DAG dependency to the DOT object.
 
     :param deps: List of DAG dependencies
     :return: Graphviz object
-    :rtype: graphviz.Digraph
     """
     dot = graphviz.Digraph(graph_attr={"rankdir": "LR"})
 
@@ -169,14 +169,13 @@ def render_dag_dependencies(deps: dict[str, list[DagDependency]]) -> graphviz.Di
 
 def render_dag(dag: DAG, tis: list[TaskInstance] | None = None) -> graphviz.Digraph:
     """
-    Renders the DAG object to the DOT object.
+    Render the DAG object to the DOT object.
 
     If an task instance list is passed, the nodes will be painted according to task statuses.
 
     :param dag: DAG that will be rendered.
     :param tis: List of task instances
     :return: Graphviz object
-    :rtype: graphviz.Digraph
     """
     dot = graphviz.Digraph(
         dag.dag_id,
