@@ -22,47 +22,43 @@ import io
 import logging
 import os
 import zipfile
-from collections import OrderedDict
 from pathlib import Path
-from typing import TYPE_CHECKING, Generator, NamedTuple, Pattern, overload
+from typing import Generator, NamedTuple, Pattern, Protocol, overload
 
 import re2
 from pathspec.patterns import GitWildMatchPattern
-from typing_extensions import Protocol
 
 from airflow.configuration import conf
 from airflow.exceptions import RemovedInAirflow3Warning
 
-if TYPE_CHECKING:
-    import pathlib
-
 log = logging.getLogger(__name__)
 
 
 class _IgnoreRule(Protocol):
-    """Interface for ignore rules for structural subtyping"""
+    """Interface for ignore rules for structural subtyping."""
 
     @staticmethod
     def compile(pattern: str, base_dir: Path, definition_file: Path) -> _IgnoreRule | None:
         """
-        Build an ignore rule from the supplied pattern where base_dir
-        and definition_file should be absolute paths.
+        Build an ignore rule from the supplied pattern.
+
+        ``base_dir`` and ``definition_file`` should be absolute paths.
         """
 
     @staticmethod
     def match(path: Path, rules: list[_IgnoreRule]) -> bool:
-        """Match a candidate absolute path against a list of rules"""
+        """Match a candidate absolute path against a list of rules."""
 
 
 class _RegexpIgnoreRule(NamedTuple):
-    """Typed namedtuple with utility functions for regexp ignore rules"""
+    """Typed namedtuple with utility functions for regexp ignore rules."""
 
     pattern: Pattern
     base_dir: Path
 
     @staticmethod
     def compile(pattern: str, base_dir: Path, definition_file: Path) -> _IgnoreRule | None:
-        """Build an ignore rule from the supplied regexp pattern and log a useful warning if it is invalid"""
+        """Build an ignore rule from the supplied regexp pattern and log a useful warning if it is invalid."""
         try:
             return _RegexpIgnoreRule(re2.compile(pattern), base_dir)
         except re2.error as e:
@@ -71,7 +67,7 @@ class _RegexpIgnoreRule(NamedTuple):
 
     @staticmethod
     def match(path: Path, rules: list[_IgnoreRule]) -> bool:
-        """Match a list of ignore rules against the supplied path"""
+        """Match a list of ignore rules against the supplied path."""
         for rule in rules:
             if not isinstance(rule, _RegexpIgnoreRule):
                 raise ValueError(f"_RegexpIgnoreRule cannot match rules of type: {type(rule)}")
@@ -81,7 +77,7 @@ class _RegexpIgnoreRule(NamedTuple):
 
 
 class _GlobIgnoreRule(NamedTuple):
-    """Typed namedtuple with utility functions for glob ignore rules"""
+    """Typed namedtuple with utility functions for glob ignore rules."""
 
     pattern: Pattern
     raw_pattern: str
@@ -90,7 +86,7 @@ class _GlobIgnoreRule(NamedTuple):
 
     @staticmethod
     def compile(pattern: str, _, definition_file: Path) -> _IgnoreRule | None:
-        """Build an ignore rule from the supplied glob pattern and log a useful warning if it is invalid"""
+        """Build an ignore rule from the supplied glob pattern and log a useful warning if it is invalid."""
         relative_to: Path | None = None
         if pattern.strip() == "/":
             # "/" doesn't match anything in gitignore
@@ -107,7 +103,7 @@ class _GlobIgnoreRule(NamedTuple):
 
     @staticmethod
     def match(path: Path, rules: list[_IgnoreRule]) -> bool:
-        """Match a list of ignore rules against the supplied path"""
+        """Match a list of ignore rules against the supplied path."""
         matched = False
         for r in rules:
             if not isinstance(r, _GlobIgnoreRule):
@@ -123,7 +119,7 @@ class _GlobIgnoreRule(NamedTuple):
 
 
 def TemporaryDirectory(*args, **kwargs):
-    """This function is deprecated. Please use `tempfile.TemporaryDirectory`"""
+    """Use `tempfile.TemporaryDirectory`, this function is deprecated."""
     import warnings
     from tempfile import TemporaryDirectory as TmpDir
 
@@ -138,8 +134,9 @@ def TemporaryDirectory(*args, **kwargs):
 
 def mkdirs(path, mode):
     """
-    Creates the directory specified by path, creating intermediate directories
-    as necessary. If directory already exists, this is a no-op.
+    Create the directory specified by path, creating intermediate directories as necessary.
+
+    If directory already exists, this is a no-op.
 
     :param path: The directory to create
     :param mode: The mode to give to the directory e.g. 0o755, ignores umask
@@ -168,10 +165,7 @@ def correct_maybe_zipped(fileloc: str | Path) -> str | Path:
 
 
 def correct_maybe_zipped(fileloc: None | str | Path) -> None | str | Path:
-    """
-    If the path contains a folder with a .zip suffix, then
-    the folder is treated as a zip archive and path to zip is returned.
-    """
+    """If the path contains a folder with a .zip suffix, treat it as a zip archive and return path."""
     if not fileloc:
         return fileloc
     search_ = ZIP_REGEX.search(str(fileloc))
@@ -186,8 +180,10 @@ def correct_maybe_zipped(fileloc: None | str | Path) -> None | str | Path:
 
 def open_maybe_zipped(fileloc, mode="r"):
     """
-    Opens the given file. If the path contains a folder with a .zip suffix, then
-    the folder is treated as a zip archive, opening the file inside the archive.
+    Open the given file.
+
+    If the path contains a folder with a .zip suffix, then the folder
+    is treated as a zip archive, opening the file inside the archive.
 
     :return: a file object, as in `open`, or as in `ZipFile.open`.
     """
@@ -199,13 +195,12 @@ def open_maybe_zipped(fileloc, mode="r"):
 
 
 def _find_path_from_directory(
-    base_dir_path: str,
+    base_dir_path: str | os.PathLike[str],
     ignore_file_name: str,
     ignore_rule_type: type[_IgnoreRule],
 ) -> Generator[str, None, None]:
-    """
-    Recursively search the base path and return the list of file paths that should not be ignored by
-    regular expressions in any ignore files at each directory level.
+    """Recursively search the base path and return the list of file paths that should not be ignored.
+
     :param base_dir_path: the base path to be searched
     :param ignore_file_name: the file name containing regular expressions for files that should be ignored.
     :param ignore_rule_type: the concrete class for ignore rules, which implements the _IgnoreRule interface.
@@ -234,7 +229,7 @@ def _find_path_from_directory(
                 ]
                 # evaluation order of patterns is important with negation
                 # so that later patterns can override earlier patterns
-                patterns = list(OrderedDict.fromkeys(patterns).keys())
+                patterns = list(dict.fromkeys(patterns))
 
         dirs[:] = [subdir for subdir in dirs if not ignore_rule_type.match(Path(root) / subdir, patterns)]
 
@@ -258,12 +253,12 @@ def _find_path_from_directory(
 
 
 def find_path_from_directory(
-    base_dir_path: str,
+    base_dir_path: str | os.PathLike[str],
     ignore_file_name: str,
     ignore_file_syntax: str = conf.get_mandatory_value("core", "DAG_IGNORE_FILE_SYNTAX", fallback="regexp"),
 ) -> Generator[str, None, None]:
-    """
-    Recursively search the base path and return the list of file paths that should not be ignored.
+    """Recursively search the base path for a list of file paths that should not be ignored.
+
     :param base_dir_path: the base path to be searched
     :param ignore_file_name: the file name in which specifies the patterns of files/dirs to be ignored
     :param ignore_file_syntax: the syntax of patterns in the ignore file: regexp or glob
@@ -279,12 +274,11 @@ def find_path_from_directory(
 
 
 def list_py_file_paths(
-    directory: str | pathlib.Path,
+    directory: str | os.PathLike[str] | None,
     safe_mode: bool = conf.getboolean("core", "DAG_DISCOVERY_SAFE_MODE", fallback=True),
     include_examples: bool | None = None,
 ) -> list[str]:
-    """
-    Traverse a directory and look for Python files.
+    """Traverse a directory and look for Python files.
 
     :param directory: the directory to traverse
     :param safe_mode: whether to use a heuristic to determine whether a file
@@ -306,16 +300,16 @@ def list_py_file_paths(
     if include_examples:
         from airflow import example_dags
 
-        example_dag_folder = example_dags.__path__[0]  # type: ignore
+        example_dag_folder = next(iter(example_dags.__path__))
         file_paths.extend(list_py_file_paths(example_dag_folder, safe_mode, include_examples=False))
     return file_paths
 
 
-def find_dag_file_paths(directory: str | pathlib.Path, safe_mode: bool) -> list[str]:
-    """Finds file paths of all DAG files."""
+def find_dag_file_paths(directory: str | os.PathLike[str], safe_mode: bool) -> list[str]:
+    """Find file paths of all DAG files."""
     file_paths = []
 
-    for file_path in find_path_from_directory(str(directory), ".airflowignore"):
+    for file_path in find_path_from_directory(directory, ".airflowignore"):
         try:
             if not os.path.isfile(file_path):
                 continue
@@ -338,6 +332,7 @@ COMMENT_PATTERN = re2.compile(r"\s*#.*")
 def might_contain_dag(file_path: str, safe_mode: bool, zip_file: zipfile.ZipFile | None = None) -> bool:
     """
     Check whether a Python file contains Airflow DAGs.
+
     When safe_mode is off (with False value), this function always returns True.
 
     If might_contain_dag_callable isn't specified, it uses airflow default heuristic
