@@ -15,48 +15,40 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
+
 import copy
+import itertools
 import re
 import signal
 import warnings
 from datetime import datetime
 from functools import reduce
-from itertools import filterfalse, tee
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Dict,
-    Generator,
-    Iterable,
-    List,
-    MutableMapping,
-    Optional,
-    Tuple,
-    TypeVar,
-    cast,
-)
+from typing import TYPE_CHECKING, Any, Callable, Generator, Iterable, Mapping, MutableMapping, TypeVar, cast
+
+from lazy_object_proxy import Proxy
 
 from airflow.configuration import conf
-from airflow.exceptions import AirflowException
+from airflow.exceptions import AirflowException, RemovedInAirflow3Warning
 from airflow.utils.context import Context
 from airflow.utils.module_loading import import_string
+from airflow.utils.types import NOTSET
 
 if TYPE_CHECKING:
     import jinja2
 
     from airflow.models.taskinstance import TaskInstance
 
-KEY_REGEX = re.compile(r'^[\w.-]+$')
-GROUP_KEY_REGEX = re.compile(r'^[\w-]+$')
-CAMELCASE_TO_SNAKE_CASE_REGEX = re.compile(r'(?!^)([A-Z]+)')
+KEY_REGEX = re.compile(r"^[\w.-]+$")
+GROUP_KEY_REGEX = re.compile(r"^[\w-]+$")
+CAMELCASE_TO_SNAKE_CASE_REGEX = re.compile(r"(?!^)([A-Z]+)")
 
-T = TypeVar('T')
-S = TypeVar('S')
+T = TypeVar("T")
+S = TypeVar("S")
 
 
 def validate_key(k: str, max_length: int = 250):
-    """Validates value used as a key."""
+    """Validate value used as a key."""
     if not isinstance(k, str):
         raise TypeError(f"The key has to be a string and is {type(k)}:{k}")
     if len(k) > max_length:
@@ -69,7 +61,7 @@ def validate_key(k: str, max_length: int = 250):
 
 
 def validate_group_key(k: str, max_length: int = 200):
-    """Validates value used as a group key."""
+    """Validate value used as a group key."""
     if not isinstance(k, str):
         raise TypeError(f"The key has to be a string and is {type(k)}:{k}")
     if len(k) > max_length:
@@ -80,8 +72,8 @@ def validate_group_key(k: str, max_length: int = 200):
         )
 
 
-def alchemy_to_dict(obj: Any) -> Optional[Dict]:
-    """Transforms a SQLAlchemy model instance into a dictionary"""
+def alchemy_to_dict(obj: Any) -> dict | None:
+    """Transform a SQLAlchemy model instance into a dictionary."""
     if not obj:
         return None
     output = {}
@@ -93,10 +85,10 @@ def alchemy_to_dict(obj: Any) -> Optional[Dict]:
     return output
 
 
-def ask_yesno(question: str, default: Optional[bool] = None) -> bool:
-    """Helper to get a yes or no answer from the user."""
-    yes = {'yes', 'y'}
-    no = {'no', 'n'}
+def ask_yesno(question: str, default: bool | None = None) -> bool:
+    """Get a yes or no answer from the user."""
+    yes = {"yes", "y"}
+    no = {"no", "n"}
 
     print(question)
     while True:
@@ -110,8 +102,8 @@ def ask_yesno(question: str, default: Optional[bool] = None) -> bool:
         print("Please respond with y/yes or n/no.")
 
 
-def prompt_with_timeout(question: str, timeout: int, default: Optional[bool] = None) -> bool:
-    """Ask the user a question and timeout if they don't respond"""
+def prompt_with_timeout(question: str, timeout: int, default: bool | None = None) -> bool:
+    """Ask the user a question and timeout if they don't respond."""
 
     def handler(signum, frame):
         raise AirflowException(f"Timeout {timeout}s reached")
@@ -125,44 +117,43 @@ def prompt_with_timeout(question: str, timeout: int, default: Optional[bool] = N
 
 
 def is_container(obj: Any) -> bool:
-    """Test if an object is a container (iterable) but not a string"""
-    return hasattr(obj, '__iter__') and not isinstance(obj, str)
+    """Test if an object is a container (iterable) but not a string."""
+    if isinstance(obj, Proxy):
+        # Proxy of any object is considered a container because it implements __iter__
+        # to forward the call to the lazily initialized object
+        # Unwrap Proxy before checking __iter__ to evaluate the proxied object
+        obj = obj.__wrapped__
+    return hasattr(obj, "__iter__") and not isinstance(obj, str)
 
 
 def as_tuple(obj: Any) -> tuple:
-    """
-    If obj is a container, returns obj as a tuple.
-    Otherwise, returns a tuple containing obj.
-    """
+    """Return obj as a tuple if obj is a container, otherwise return a tuple containing obj."""
     if is_container(obj):
         return tuple(obj)
     else:
         return tuple([obj])
 
 
-def chunks(items: List[T], chunk_size: int) -> Generator[List[T], None, None]:
-    """Yield successive chunks of a given size from a list of items"""
+def chunks(items: list[T], chunk_size: int) -> Generator[list[T], None, None]:
+    """Yield successive chunks of a given size from a list of items."""
     if chunk_size <= 0:
-        raise ValueError('Chunk size must be a positive integer')
+        raise ValueError("Chunk size must be a positive integer")
     for i in range(0, len(items), chunk_size):
         yield items[i : i + chunk_size]
 
 
-def reduce_in_chunks(fn: Callable[[S, List[T]], S], iterable: List[T], initializer: S, chunk_size: int = 0):
-    """
-    Reduce the given list of items by splitting it into chunks
-    of the given size and passing each chunk through the reducer
-    """
-    if len(iterable) == 0:
+def reduce_in_chunks(fn: Callable[[S, list[T]], S], iterable: list[T], initializer: S, chunk_size: int = 0):
+    """Split the list of items into chunks of a given size and pass each chunk through the reducer."""
+    if not iterable:
         return initializer
     if chunk_size == 0:
         chunk_size = len(iterable)
     return reduce(fn, chunks(iterable, chunk_size), initializer)
 
 
-def as_flattened_list(iterable: Iterable[Iterable[T]]) -> List[T]:
+def as_flattened_list(iterable: Iterable[Iterable[T]]) -> list[T]:
     """
-    Return an iterable with one level flattened
+    Return an iterable with one level flattened.
 
     >>> as_flattened_list((('blue', 'red'), ('green', 'yellow', 'pink')))
     ['blue', 'red', 'green', 'yellow', 'pink']
@@ -170,8 +161,8 @@ def as_flattened_list(iterable: Iterable[Iterable[T]]) -> List[T]:
     return [e for i in iterable for e in i]
 
 
-def parse_template_string(template_string: str) -> Tuple[Optional[str], Optional["jinja2.Template"]]:
-    """Parses Jinja template string."""
+def parse_template_string(template_string: str) -> tuple[str | None, jinja2.Template | None]:
+    """Parse Jinja template string."""
     import jinja2
 
     if "{{" in template_string:  # jinja mode
@@ -180,10 +171,9 @@ def parse_template_string(template_string: str) -> Tuple[Optional[str], Optional
         return template_string, None
 
 
-def render_log_filename(ti: "TaskInstance", try_number, filename_template) -> str:
+def render_log_filename(ti: TaskInstance, try_number, filename_template) -> str:
     """
-    Given task instance, try_number, filename_template, return the rendered log
-    filename
+    Given task instance, try_number, filename_template, return the rendered log filename.
 
     :param ti: task instance
     :param try_number: try_number of the task
@@ -193,7 +183,7 @@ def render_log_filename(ti: "TaskInstance", try_number, filename_template) -> st
     filename_template, filename_jinja_template = parse_template_string(filename_template)
     if filename_jinja_template:
         jinja_context = ti.get_template_context()
-        jinja_context['try_number'] = try_number
+        jinja_context["try_number"] = try_number
         return render_template_to_string(filename_jinja_template, jinja_context)
 
     return filename_template.format(
@@ -205,11 +195,11 @@ def render_log_filename(ti: "TaskInstance", try_number, filename_template) -> st
 
 
 def convert_camel_to_snake(camel_str: str) -> str:
-    """Converts CamelCase to snake_case."""
-    return CAMELCASE_TO_SNAKE_CASE_REGEX.sub(r'_\1', camel_str).lower()
+    """Convert CamelCase to snake_case."""
+    return CAMELCASE_TO_SNAKE_CASE_REGEX.sub(r"_\1", camel_str).lower()
 
 
-def merge_dicts(dict1: Dict, dict2: Dict) -> Dict:
+def merge_dicts(dict1: dict, dict2: dict) -> dict:
     """
     Merge two dicts recursively, returning new dict (input dict is not mutated).
 
@@ -224,41 +214,42 @@ def merge_dicts(dict1: Dict, dict2: Dict) -> Dict:
     return merged
 
 
-def partition(pred: Callable[[T], bool], iterable: Iterable[T]) -> Tuple[Iterable[T], Iterable[T]]:
-    """Use a predicate to partition entries into false entries and true entries"""
-    iter_1, iter_2 = tee(iterable)
-    return filterfalse(pred, iter_1), filter(pred, iter_2)
+def partition(pred: Callable[[T], bool], iterable: Iterable[T]) -> tuple[Iterable[T], Iterable[T]]:
+    """Use a predicate to partition entries into false entries and true entries."""
+    iter_1, iter_2 = itertools.tee(iterable)
+    return itertools.filterfalse(pred, iter_1), filter(pred, iter_2)
 
 
 def chain(*args, **kwargs):
-    """This function is deprecated. Please use `airflow.models.baseoperator.chain`."""
+    """Use `airflow.models.baseoperator.chain`, this function is deprecated."""
     warnings.warn(
         "This function is deprecated. Please use `airflow.models.baseoperator.chain`.",
-        DeprecationWarning,
+        RemovedInAirflow3Warning,
         stacklevel=2,
     )
-    return import_string('airflow.models.baseoperator.chain')(*args, **kwargs)
+    return import_string("airflow.models.baseoperator.chain")(*args, **kwargs)
 
 
 def cross_downstream(*args, **kwargs):
-    """This function is deprecated. Please use `airflow.models.baseoperator.cross_downstream`."""
+    """Use `airflow.models.baseoperator.cross_downstream`, this function is deprecated."""
     warnings.warn(
         "This function is deprecated. Please use `airflow.models.baseoperator.cross_downstream`.",
-        DeprecationWarning,
+        RemovedInAirflow3Warning,
         stacklevel=2,
     )
-    return import_string('airflow.models.baseoperator.cross_downstream')(*args, **kwargs)
+    return import_string("airflow.models.baseoperator.cross_downstream")(*args, **kwargs)
 
 
-def build_airflow_url_with_query(query: Dict[str, Any]) -> str:
+def build_airflow_url_with_query(query: dict[str, Any]) -> str:
     """
-    Build airflow url using base_url and default_view and provided query
+    Build airflow url using base_url and default_view and provided query.
+
     For example:
-    'http://0.0.0.0:8000/base/graph?dag_id=my-task&root=&execution_date=2020-10-27T10%3A59%3A25.615587
+    http://0.0.0.0:8000/base/graph?dag_id=my-task&root=&execution_date=2020-10-27T10%3A59%3A25.615587
     """
     import flask
 
-    view = conf.get_mandatory_value('webserver', 'dag_default_view').lower()
+    view = conf.get_mandatory_value("webserver", "dag_default_view").lower()
     return flask.url_for(f"Airflow.{view}", **query)
 
 
@@ -293,19 +284,19 @@ def render_template(template: Any, context: MutableMapping[str, Any], *, native:
     return "".join(nodes)
 
 
-def render_template_to_string(template: "jinja2.Template", context: Context) -> str:
+def render_template_to_string(template: jinja2.Template, context: Context) -> str:
     """Shorthand to ``render_template(native=False)`` with better typing support."""
     return render_template(template, cast(MutableMapping[str, Any], context), native=False)
 
 
-def render_template_as_native(template: "jinja2.Template", context: Context) -> Any:
+def render_template_as_native(template: jinja2.Template, context: Context) -> Any:
     """Shorthand to ``render_template(native=True)`` with better typing support."""
     return render_template(template, cast(MutableMapping[str, Any], context), native=True)
 
 
 def exactly_one(*args) -> bool:
     """
-    Returns True if exactly one of *args is "truthy", and False otherwise.
+    Return True if exactly one of *args is "truthy", and False otherwise.
 
     If user supplies an iterable, we raise ValueError and force them to unpack.
     """
@@ -316,10 +307,27 @@ def exactly_one(*args) -> bool:
     return sum(map(bool, args)) == 1
 
 
-def prune_dict(val: Any, mode='strict'):
+def at_most_one(*args) -> bool:
     """
-    Given dict ``val``, returns new dict based on ``val`` with all
-    empty elements removed.
+    Return True if at most one of *args is "truthy", and False otherwise.
+
+    NOTSET is treated the same as None.
+
+    If user supplies an iterable, we raise ValueError and force them to unpack.
+    """
+
+    def is_set(val):
+        if val is NOTSET:
+            return False
+        else:
+            return bool(val)
+
+    return sum(map(is_set, args)) in (0, 1)
+
+
+def prune_dict(val: Any, mode="strict"):
+    """
+    Given dict ``val``, returns new dict based on ``val`` with all empty elements removed.
 
     What constitutes "empty" is controlled by the ``mode`` parameter.  If mode is 'strict'
     then only ``None`` elements will be removed.  If mode is ``truthy``, then element ``x``
@@ -327,9 +335,9 @@ def prune_dict(val: Any, mode='strict'):
     """
 
     def is_empty(x):
-        if mode == 'strict':
+        if mode == "strict":
             return x is None
-        elif mode == 'truthy':
+        elif mode == "truthy":
             return bool(x) is False
         raise ValueError("allowable values for `mode` include 'truthy' and 'strict'")
 
@@ -340,7 +348,7 @@ def prune_dict(val: Any, mode='strict'):
                 continue
             elif isinstance(v, (list, dict)):
                 new_val = prune_dict(v, mode=mode)
-                if new_val:
+                if not is_empty(new_val):
                     new_dict[k] = new_val
             else:
                 new_dict[k] = v
@@ -352,10 +360,24 @@ def prune_dict(val: Any, mode='strict'):
                 continue
             elif isinstance(v, (list, dict)):
                 new_val = prune_dict(v, mode=mode)
-                if new_val:
+                if not is_empty(new_val):
                     new_list.append(new_val)
             else:
                 new_list.append(v)
         return new_list
     else:
         return val
+
+
+def prevent_duplicates(kwargs1: dict[str, Any], kwargs2: Mapping[str, Any], *, fail_reason: str) -> None:
+    """Ensure *kwargs1* and *kwargs2* do not contain common keys.
+
+    :raises TypeError: If common keys are found.
+    """
+    duplicated_keys = set(kwargs1).intersection(kwargs2)
+    if not duplicated_keys:
+        return
+    if len(duplicated_keys) == 1:
+        raise TypeError(f"{fail_reason} argument: {duplicated_keys.pop()}")
+    duplicated_keys_display = ", ".join(sorted(duplicated_keys))
+    raise TypeError(f"{fail_reason} arguments: {duplicated_keys_display}")
