@@ -22,21 +22,22 @@ import pendulum
 
 
 class TimezoneAware(logging.Formatter):
-    """
-    Override `default_time_format`, `default_msec_format` and `formatTime` to specify utc offset.
-    utc offset is the matter, without it, time conversion could be wrong.
-    With this Formatter, `%(asctime)s` will be formatted containing utc offset. (ISO 8601)
-    (e.g. 2022-06-12T13:00:00.123+0000)
+    """Override time-formatting methods to include UTC offset.
+
+    Since Airflow parses the logs to perform time conversion, UTC offset is
+    critical information. This formatter ensures ``%(asctime)s`` is formatted
+    containing the offset in ISO 8601, e.g. ``2022-06-12T13:00:00.123+0000``.
     """
 
-    default_time_format = '%Y-%m-%dT%H:%M:%S'
-    default_msec_format = '%s.%03d'
-    default_tz_format = '%z'
+    default_time_format = "%Y-%m-%dT%H:%M:%S"
+    default_msec_format = "%s.%03d"
+    default_tz_format = "%z"
 
     def formatTime(self, record, datefmt=None):
-        """
-        Returns the creation time of the specified LogRecord in ISO 8601 date and time format
-        in the local time zone.
+        """Format time in record.
+
+        This returns the creation time of the specified LogRecord in ISO 8601
+        date and time format in the local time zone.
         """
         dt = pendulum.from_timestamp(record.created, tz=pendulum.local_timezone())
         if datefmt:
