@@ -25,8 +25,9 @@ from typing import Callable
 
 def import_string(dotted_path: str):
     """
-    Import a dotted module path and return the attribute/class designated by the
-    last name in the path. Raise ImportError if the import failed.
+    Import a dotted module path and return the attribute/class designated by the last name in the path.
+
+    Raise ImportError if the import failed.
     """
     try:
         module_path, class_name = dotted_path.rsplit(".", 1)
@@ -43,7 +44,7 @@ def import_string(dotted_path: str):
 
 def qualname(o: object | Callable) -> str:
     """Convert an attribute/class/function to a string importable by ``import_string``."""
-    if callable(o):
+    if callable(o) and hasattr(o, "__module__") and hasattr(o, "__name__"):
         return f"{o.__module__}.{o.__name__}"
 
     cls = o
