@@ -15,7 +15,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Utilities for creating a virtual environment"""
+"""Utilities for creating a virtual environment."""
 from __future__ import annotations
 
 import os
@@ -29,25 +29,25 @@ from airflow.utils.process_utils import execute_in_subprocess
 
 
 def _generate_virtualenv_cmd(tmp_dir: str, python_bin: str, system_site_packages: bool) -> list[str]:
-    cmd = [sys.executable, '-m', 'virtualenv', tmp_dir]
+    cmd = [sys.executable, "-m", "virtualenv", tmp_dir]
     if system_site_packages:
-        cmd.append('--system-site-packages')
+        cmd.append("--system-site-packages")
     if python_bin is not None:
-        cmd.append(f'--python={python_bin}')
+        cmd.append(f"--python={python_bin}")
     return cmd
 
 
 def _generate_pip_install_cmd_from_file(
     tmp_dir: str, requirements_file_path: str, pip_install_options: list[str]
 ) -> list[str]:
-    cmd = [f'{tmp_dir}/bin/pip', 'install'] + pip_install_options + ['-r']
+    cmd = [f"{tmp_dir}/bin/pip", "install"] + pip_install_options + ["-r"]
     return cmd + [requirements_file_path]
 
 
 def _generate_pip_install_cmd_from_list(
     tmp_dir: str, requirements: list[str], pip_install_options: list[str]
 ) -> list[str]:
-    cmd = [f'{tmp_dir}/bin/pip', 'install'] + pip_install_options
+    cmd = [f"{tmp_dir}/bin/pip", "install"] + pip_install_options
     return cmd + requirements
 
 
@@ -68,7 +68,8 @@ def prepare_virtualenv(
     requirements_file_path: str | None = None,
     pip_install_options: list[str] | None = None,
 ) -> str:
-    """Creates a virtual environment and installs the additional python packages.
+    """
+    Create a virtual environment and install the additional python packages.
 
     :param venv_directory: The path for directory where the environment will be created.
     :param python_bin: Path to the Python executable.
@@ -77,7 +78,6 @@ def prepare_virtualenv(
     :param requirements: List of additional python packages.
     :param requirements_file_path: Path to the ``requirements.txt`` file.
     :return: Path to a binary file with Python in a virtual environment.
-    :rtype: str
     """
     if pip_install_options is None:
         pip_install_options = []
@@ -99,7 +99,7 @@ def prepare_virtualenv(
     if pip_cmd:
         execute_in_subprocess(pip_cmd)
 
-    return f'{venv_directory}/bin/python'
+    return f"{venv_directory}/bin/python"
 
 
 def write_python_script(
@@ -108,7 +108,7 @@ def write_python_script(
     render_template_as_native_obj: bool = False,
 ):
     """
-    Renders the python script to a file to execute in the virtual environment.
+    Render the python script to a file to execute in the virtual environment.
 
     :param jinja_context: The jinja context variables to unpack and replace with its placeholders in the
         template file.
@@ -124,5 +124,5 @@ def write_python_script(
         )
     else:
         template_env = jinja2.Environment(loader=template_loader, undefined=jinja2.StrictUndefined)
-    template = template_env.get_template('python_virtualenv_script.jinja2')
+    template = template_env.get_template("python_virtualenv_script.jinja2")
     template.stream(**jinja_context).dump(filename)
