@@ -14,13 +14,16 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Get the ISO standard day number of the week from a given day string"""
+"""Get the ISO standard day number of the week from a given day string."""
+from __future__ import annotations
+
 import enum
+from typing import Iterable
 
 
 @enum.unique
 class WeekDay(enum.IntEnum):
-    """Python Enum containing Days of the Week"""
+    """Python Enum containing Days of the Week."""
 
     MONDAY = 1
     TUESDAY = 2
@@ -31,18 +34,43 @@ class WeekDay(enum.IntEnum):
     SUNDAY = 7
 
     @classmethod
-    def get_weekday_number(cls, week_day_str):
+    def get_weekday_number(cls, week_day_str: str):
         """
-        Return the ISO Week Day Number for a Week Day
+        Return the ISO Week Day Number for a Week Day.
 
         :param week_day_str: Full Name of the Week Day. Example: "Sunday"
-        :type week_day_str: str
         :return: ISO Week Day Number corresponding to the provided Weekday
         """
         sanitized_week_day_str = week_day_str.upper()
 
-        # pylint: disable=unsupported-membership-test
         if sanitized_week_day_str not in cls.__members__:
             raise AttributeError(f'Invalid Week Day passed: "{week_day_str}"')
 
         return cls[sanitized_week_day_str]
+
+    @classmethod
+    def convert(cls, day: str | WeekDay) -> int:
+        """Return the day number in the week."""
+        if isinstance(day, WeekDay):
+            return day
+        return cls.get_weekday_number(week_day_str=day)
+
+    @classmethod
+    def validate_week_day(
+        cls,
+        week_day: str | WeekDay | Iterable[str] | Iterable[WeekDay],
+    ) -> set[int]:
+        """Validate each item of iterable and create a set to ease compare of values."""
+        if not isinstance(week_day, Iterable):
+            if isinstance(week_day, WeekDay):
+                week_day = {week_day}
+            else:
+                raise TypeError(
+                    f"Unsupported Type for week_day parameter: {type(week_day)}."
+                    "Input should be iterable type:"
+                    "str, set, list, dict or Weekday enum type"
+                )
+        if isinstance(week_day, str):
+            week_day = {week_day}
+
+        return {cls.convert(item) for item in week_day}
