@@ -19,19 +19,20 @@ from __future__ import annotations
 import logging
 from importlib import import_module
 
-from flask import g, redirect, url_for
-from flask_login import logout_user
+from flask import g, redirect
 
 from airflow.configuration import conf
 from airflow.exceptions import AirflowConfigException, AirflowException
+from airflow.www.extensions.init_auth_manager import get_auth_manager
 
 log = logging.getLogger(__name__)
 
 
 def init_xframe_protection(app):
     """
-    Add X-Frame-Options header. Use it to avoid click-jacking attacks, by ensuring that their content is not
-    embedded into other sites.
+    Add X-Frame-Options header.
+
+    Use it to avoid click-jacking attacks, by ensuring that their content is not embedded into other sites.
 
     See also: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Frame-Options
     """
@@ -47,7 +48,7 @@ def init_xframe_protection(app):
 
 
 def init_api_experimental_auth(app):
-    """Loads authentication backends"""
+    """Loads authentication backends."""
     auth_backends = "airflow.api.auth.backend.default"
     try:
         auth_backends = conf.get("api", "auth_backends")
@@ -68,6 +69,5 @@ def init_api_experimental_auth(app):
 def init_check_user_active(app):
     @app.before_request
     def check_user_active():
-        if g.user is not None and not g.user.is_anonymous and not g.user.is_active:
-            logout_user()
-            return redirect(url_for(app.appbuilder.sm.auth_view.endpoint + ".login"))
+        if get_auth_manager().is_logged_in() and not g.user.is_active:
+            return redirect(get_auth_manager().get_url_logout())
